#include "mainwindow.h"
#include "ui_mainwindow.h"
#include <QApplication>
#include <QNetworkAccessManager>
#include <QNetworkReply>
#include <QDebug>
#include <QtSql>

#include "constants.h"
#include "dialognewstock.h"
#include "stutility.h"

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    bool result = connectDb();
    if(result){
        loadAllStock();
    }
    qDebug() << "connect db:";
    qDebug() << result;
}

MainWindow::~MainWindow()
{
    delete ui;
}

bool MainWindow::connectDb()
{
    QSqlDatabase db = QSqlDatabase::addDatabase("QSQLITE");
    //QSqlDatabase db = QSqlDatabase::addDatabase("QMYSQL");
    //db.setHostName(host);
    //db.setDatabaseName("/Users/walle/Documents/qt-projects/GameDevKit/appIconDB.db");
    //db.setDatabaseName(dbname);
    //db.setDatabaseName(":memory:");
    QDir targetDir(QStandardPaths::writableLocation(QStandardPaths::DataLocation));
    if(!targetDir.exists()){    /**< 如果目标目录不存在，则进行创建 */
        if(!targetDir.mkdir(targetDir.absolutePath()))
            return false;
    }
    db.setDatabaseName(QStandardPaths::writableLocation(QStandardPaths::DataLocation) + QDir::separator()  +  dbname);
    qDebug() << QStandardPaths::writableLocation(QStandardPaths::DataLocation);
    //db.setUserName(user);
    //db.setPassword(pwd);

    if(!db.open())
    {
        qDebug()<<"Database Error!"<<db.lastError().text();
        return false;
    }else{
        qDebug()<<"Database success!";
        QSqlQuery query; //以下执行相关SQL语句
        query.exec("CREATE TABLE if not exists stockHistory (historyId INTEGER PRIMARY KEY AUTOINCREMENT,stockId CHAR (20),date CHAR (40), close FLOAT, volume FLOAT, open FLOAT, high FLOAT, low FLOAT);");
        query.exec("CREATE TABLE if not exists stockname (stockId INT, stockName CHAR (40));");
        return true;
    }
}


void MainWindow::on_btnStartGet_clicked()
{
    if(ui->editStockId->text().trimmed().length() == 0){
        qDebug() << "stock Id can not be null";
        return;
    }

    //qDebug() << "yahoo:" << getYAHOOURLString(ui->editStockId->text());
    //return;
    QNetworkAccessManager *manager = new QNetworkAccessManager(this);
    connect(manager,SIGNAL(finished(QNetworkReply*)),this, SLOT(query(QNetworkReply*)));
    //manager->get(QNetworkRequest(QUrl(stockSource)));
    QNetworkRequest request(QUrl(STUtility::getYAHOOURLString(ui->editStockId->text())));

    qDebug() << QUrl(STUtility::getYAHOOURLString(ui->editStockId->text())).toString();

    request.setHeader(QNetworkRequest::ContentTypeHeader,
        "application/x-www-form-urlencoded");
    //QByteArray postData;
    //postData.append(" 5d|false|BIDU");

//    QUrlQuery postData;
//    postData.addQueryItem("", "5d|false|BIDU");
//    manager->post(request, postData.toString(QUrl::FullyEncoded).toUtf8());

//    manager->post(request, postData);

    manager->get(request);

}

void MainWindow::query(QNetworkReply* reply){
    const int eachCount = 200;
    vector<StockHistory> websiteHistorys(eachCount);

    QString input = reply->readAll();
    qDebug() << input;
    qDebug() << "request finish!";

    QRegularExpression dateRegex("(?:<td class=\"yfnc_tabledata1\" nowrap align=\"right\">)(.*?)(?:</td>)");
    QRegularExpressionMatchIterator dateItr = dateRegex.globalMatch(input);

    int dateCount = 0;

    QDate databaseMaxDate = STUtility::getMaxHistoryDate(ui->editStockId->text());

    //QDate databaseMaxDate = QDate(2015,1,1);

    //save date
    while (dateItr.hasNext()) {
        if(dateCount == eachCount){
           break;
        }
        QRegularExpressionMatch match = dateItr.next();
        if (match.hasMatch()) {
             qDebug() << "date:" << match.captured(1);
             QString dateString = match.captured(1);

             QDate currentDate = QDate::fromString(STUtility::getValideDate(dateString),"MM dd, yyyy");
             if(currentDate > databaseMaxDate){
                 websiteHistorys[dateCount] = StockHistory();
                 websiteHistorys[dateCount].setDate(currentDate);
                 ++dateCount;
             }else{
                 break;
             }
        }
    }
    websiteHistorys.resize(dateCount);
    //save price
    QRegularExpression priceRegex("(?:<td class=\"yfnc_tabledata1\" align=\"right\">)(.*?)(?:</td>)");
    QRegularExpressionMatchIterator priceItr = priceRegex.globalMatch(input);

    int priceTypeIndex = 0;
    int priceIndex = 0;
    while (priceItr.hasNext()) {
        if(priceIndex == dateCount){
           break;
        }
        QRegularExpressionMatch match = priceItr.next();
        if (match.hasMatch()) {
             qDebug() << "price:" << match.captured(1);

             if(priceTypeIndex == 0){
                 float price = match.captured(1).toFloat();
                 websiteHistorys[priceIndex].setOpen(price);
             }else if(priceTypeIndex == 1){
                 float price = match.captured(1).toFloat();
                 websiteHistorys[priceIndex].setHigh(price);
             }else if(priceTypeIndex == 2){
                 float price = match.captured(1).toFloat();
                 websiteHistorys[priceIndex].setLow(price);
             }else if(priceTypeIndex == 3){
                 float price = match.captured(1).toFloat();
                 websiteHistorys[priceIndex].setClose(price);
             }else if(priceTypeIndex == 4){
                 std::string volumeString = match.captured(1).toStdString();
                 QString qVolumeString = QString(volumeString.c_str());
                 qVolumeString = qVolumeString.replace(",", "");
                 websiteHistorys[priceIndex].setVolume(qVolumeString.toInt());
             }
//             else if(priceTypeIndex == 5){
//                 //do nothing
//             }

             ++priceTypeIndex;
             if(priceTypeIndex == 6){
                 priceTypeIndex = 0;
                 ++priceIndex;
             }

        }
    }

    // insert into database
    for(int i = 0; i < dateCount; ++i){
        StockHistory oneHistory = websiteHistorys[i];

        QSqlQuery query;
        QString qQuery = "INSERT INTO stockHistory (stockId,date,close,volume,open,high,low) "
                "VALUES (\"%1\", \"%2\", \"%3\", \"%4\", \"%5\", \"%6\", \"%7\")";
        qQuery = qQuery.arg(ui->editStockId->text()).arg(oneHistory.getDate().toString("yyyy-MM-dd")).arg(oneHistory.getClose())
                .arg(oneHistory.getVolume()).arg(oneHistory.getOpen()).arg(oneHistory.getHigh()).arg(oneHistory.getLow());

        query.prepare(qQuery);
        //qDebug() << "query:" << qQuery;
        bool result = query.exec();
        if(!result){
            qDebug() << query.lastError().text().toLocal8Bit().data();
        }else{
            qDebug() << "insert one line success";
        }
    }
}

void MainWindow::on_btnGetBestBuy_clicked()
{
    //qDebug() << getMaxHistoryDate("NTES").toString("yyyy-MM-dd");
}



void MainWindow::loadHistoryByStockId(QString stockId){
    allHistorys.clear();
    QSqlQuery query;
    QString qSelectQuery;
    qSelectQuery = QString("select historyId,stockId,date,close,volume,open,high,low from stockHistory where stockId = '%1' order by date asc").arg(stockId);
    bool execResult = query.exec(qSelectQuery);
    if(execResult){
        while(query.next()){
            StockHistory oneHistory;
            int historyId = query.value(0).toInt();
            QString stockId = query.value(1).toString();
            QDate date = query.value(2).toDate();
            float closePrice = query.value(3).toFloat();
            int volume = query.value(4).toInt();
            float openPrice = query.value(5).toFloat();
            float highPrice = query.value(6).toFloat();
            float lowPrice = query.value(7).toFloat();

            oneHistory.setStockId(stockId);
            oneHistory.setDate(date);
            oneHistory.setClose(closePrice);
            oneHistory.setVolume(volume);
            oneHistory.setOpen(openPrice);
            oneHistory.setHigh(highPrice);
            oneHistory.setLow(lowPrice);
            allHistorys.push_back(oneHistory);
        }
    }else{
        qDebug() << query.lastError().text().toLocal8Bit().data();
    }

    qDebug() << "get data ok";
}


void MainWindow::mainAnalysis(){
   float currentPrice = 0;
   qDebug() << "first buy stock";
   int action = 0;
   if(allHistorys.size() > 0){
       currentPrice = allHistorys[0].getClose();
       float longPrice = currentPrice;
       float stortPrice = currentPrice;
       float profit = 0;
       for(int i = 0; i < allHistorys.size(); ++i){
           float dayClosePrice = allHistorys[i].getClose();
           if(action == 0){
               if(dayClosePrice < currentPrice * 0.985 ){

                   currentPrice = dayClosePrice;
                   action = 1;
                   profit += (dayClosePrice - longPrice);
                   stortPrice = dayClosePrice;
                   qDebug() << "sell stock " << dayClosePrice << " and stort" << " profit " << profit;;
               }else if(dayClosePrice > currentPrice){
                   currentPrice = dayClosePrice;
                   qDebug() << "hold long stock position:" << dayClosePrice << " profit " << profit;
               }

           }else{
               if(dayClosePrice > currentPrice * 1.015 ){

                   currentPrice = dayClosePrice;
                   action = 0;
                   profit += (stortPrice - dayClosePrice);
                   longPrice = dayClosePrice;
                   qDebug() << "buy stock " << dayClosePrice << " and long" << " profit " << profit;
               }else if(dayClosePrice < currentPrice){
                   currentPrice = dayClosePrice;
                   qDebug() << "hold stort stock position:" << dayClosePrice << " profit " << profit;
               }
           }

           if(i == allHistorys.size() - 1){
               profit += (dayClosePrice - longPrice);
           }

       }

       qDebug() << "profit " << profit;

   }

}

void MainWindow::on_btnAnalysis_clicked()
{
    if(ui->editStockId->text().trimmed().length() == 0){
        qDebug() << "stock Id can not be null";
        return;
    }

    loadHistoryByStockId(ui->editStockId->text());

    mainAnalysis();
}


void MainWindow::loadAllStock(){
//    ui->comboBoxAllStock->clear();
//    QSqlQuery query;
//    QString qSelectQuery;
//    qSelectQuery = QString("select stockId,stockName from stockname");

//    bool execResult = query.exec(qSelectQuery);
//    if(execResult){
//        while(query.next()){
//            QString stockId = query.value(0).toString();
//            ui->comboBoxAllStock->addItem(stockId);
//        }
//    }else{
//        qDebug() << query.lastError().text().toLocal8Bit().data();
//    }

}

void MainWindow::on_btnNewStock_clicked()
{
    DialogNewStock* dialog = new DialogNewStock;
    dialog->show();
}

void MainWindow::on_btnRefresh_clicked()
{
    loadAllStock();
}
